import * as _codemirror_lint from '@codemirror/lint';
import { LanguageSupport } from '@codemirror/language';
import { Extension } from '@codemirror/state';
import { EditorView } from '@codemirror/view';
import { Properties } from 'csstype';
import { MapStore } from 'nanostores';

/**
 * Actions can return an object containing the two properties defined in this interface. Both are optional.
 * - update: An action can have a parameter. This method will be called whenever that parameter changes,
 *   immediately after Svelte has applied updates to the markup.
 * - destroy: Method that is called after the element is unmounted
 *
 * Additionally, you can specify which additional attributes and events the action enables on the applied element.
 * This applies to TypeScript typings only and has no effect at runtime.
 *
 * Example usage:
 * ```ts
 * interface Attributes {
 * 	newprop?: string;
 *	'on:event': (e: CustomEvent<boolean>) => void;
 * }
 *
 * export function myAction(node: HTMLElement, parameter: Parameter): ActionReturn<Parameter, Attributes> {
 *   // ...
 *   return {
 *     update: (updatedParameter) => {...},
 *     destroy: () => {...}
 *   };
 * }
 * ```
 *
 * Docs: https://svelte.dev/docs#template-syntax-element-directives-use-action
 */
interface ActionReturn<Parameter = any, Attributes extends Record<string, any> = Record<never, any>> {
    update?: (parameter: Parameter) => void;
    destroy?: () => void;
    /**
     * ### DO NOT USE THIS
     * This exists solely for type-checking and has no effect at runtime.
     * Set this through the `Attributes` generic instead.
     */
    $$_attributes?: Attributes;
}

type MaybePromise<T> = T | Promise<T>;
type Styles = {
    [val: string]: Properties | Styles;
};
type Options = {
    value: string;
    setup?: 'basic' | 'minimal';
    lang?: LanguageSupport | string;
    langMap?: Record<string, () => MaybePromise<LanguageSupport>>;
    useTabs?: boolean;
    readonly?: boolean;
    styles?: Styles;
    tabSize?: number;
    theme?: Extension;
    diagnostics?: _codemirror_lint.Diagnostic[];
    extensions?: Extension[];
    instanceStore?: MapStore<CodemirrorInstance>;
};
type CodemirrorInstance = {
    view: EditorView | null;
    extensions: Extension | null;
    value: string | null;
};
declare const withCodemirrorInstance: () => MapStore<CodemirrorInstance>;
declare const codemirror: (node: HTMLElement, options: Options) => ActionReturn<Options, {
    'on:codemirror:change': (e: CustomEvent<string>) => void;
}>;

export { codemirror, withCodemirrorInstance };
